/* Edge Impulse Arduino examples
 * Copyright (c) 2021 EdgeImpulse Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/* Includes ---------------------------------------------------------------- */
#include <ectp02_inferencing.h>
//#include <Arduino_LSM9DS1.h>
#include"LIS3DHTR.h"
#include"TFT_eSPI.h"

TFT_eSPI tft;
LIS3DHTR<TwoWire> lis;
#define CONVERT_G_TO_MS2 9.80665f
ei_impulse_result_classification_t
currentClassification[EI_CLASSIFIER_LABEL_COUNT];
const char* maxConfidenceLabel;
void runClassifier()
{
float buffer[EI_CLASSIFIER_DSP_INPUT_FRAME_SIZE] = { 0 };
for (size_t ix = 0; ix < EI_CLASSIFIER_DSP_INPUT_FRAME_SIZE; ix += 3) {
uint64_t next_tick = micros() + (EI_CLASSIFIER_INTERVAL_MS * 1000);
lis.getAcceleration(&buffer[ix], &buffer[ix + 1], &buffer[ix + 2]);
buffer[ix + 0] *= CONVERT_G_TO_MS2;
buffer[ix + 1] *= CONVERT_G_TO_MS2;
buffer[ix + 2] *= CONVERT_G_TO_MS2;
delayMicroseconds(next_tick - micros());
}
signal_t signal;
int err = numpy:: signal_from_buffer(buffer,
EI_CLASSIFIER_DSP_INPUT_FRAME_SIZE, &signal);
ei_impulse_result_t result = { 0 };
err = run_classifier(&signal, &result, false);
float maxValue = 0;
for (size_t ix = 0; ix < EI_CLASSIFIER_LABEL_COUNT; ix++) {
ei_impulse_result_classification_t classification_t =
result.classification[ix];
ei_printf(" %s: %.5f\n", classification_t.label, classification_t.value);
float value = classification_t.value;
if (value > maxValue) {
maxValue = value;
maxConfidenceLabel = classification_t.label;
}
currentClassification[ix] = classification_t;
}
}
void setup(){
tft.begin();
lis.begin(Wire1);
lis.setOutputDataRate(LIS3DHTR_DATARATE_100HZ);
lis.setFullScaleRange(LIS3DHTR_RANGE_4G);
tft.setRotation(3);
tft.setTextSize(4);
}
void loop(){
runClassifier();
tft.drawString((String)maxConfidenceLabel, 120, 120);
}




//LIS3DHTR<TwoWire> lis;
//TFT_eSPI tft;

/* Constant defines -------------------------------------------------------- */
//#define CONVERT_G_TO_MS2    9.80665f
//#define MAX_ACCEPTED_RANGE  2.0f        // starting 03/2022, models are generated setting range to +-2, but this example use Arudino library which set range to +-4g. If you are using an older model, ignore this value and use 4.0f instead

/*
 ** NOTE: If you run into TFLite arena allocation issue.
 **
 ** This may be due to may dynamic memory fragmentation.
 ** Try defining "-DEI_CLASSIFIER_ALLOCATION_STATIC" in boards.local.txt (create
 ** if it doesn't exist) and copy this file to
 ** `<ARDUINO_CORE_INSTALL_PATH>/arduino/hardware/<mbed_core>/<core_version>/`.
 **
 ** See
 ** (https://support.arduino.cc/hc/en-us/articles/360012076960-Where-are-the-installed-cores-located-)
 ** to find where Arduino installs cores on your machine.
 **
 ** If the problem persists then there's not enough memory for this model and application.
 */

/* Private variables ------------------------------------------------------- */
//static bool debug_nn = false; // Set this to true to see e.g. features generated from the raw signal

/**
* @brief      Arduino setup function
*/
//void setup()
//{
//    // put your setup code here, to run once:
//    Serial.begin(115200);
//    Serial.println("Edge Impulse Inferencing Demo");

//    if (!IMU.begin()) {
//        ei_printf("Failed to initialize IMU!\r\n");
//    }
//    else {
//        ei_printf("IMU initialized\r\n");
//    }

//    tft.begin();
//    tft.setRotation(3);
//    tft.fillScreen(TFT_WHITE);
// 
//    lis.begin(Wire1);
// 
//    if (!lis.available()) {
//    Serial.println("Failed to initialize IMU!");
//    while (1);
//    }
//    else {
//        ei_printf("IMU initialized\r\n");
//    }
//    lis.setOutputDataRate(LIS3DHTR_DATARATE_100HZ); // Setting output data rage to 25Hz, can be set up tp 5kHz 
//    lis.setFullScaleRange(LIS3DHTR_RANGE_16G); // Setting scale range to 2g, select from 2,4,8,16g
//
//
//    if (EI_CLASSIFIER_RAW_SAMPLES_PER_FRAME != 3) {
//        ei_printf("ERR: EI_CLASSIFIER_RAW_SAMPLES_PER_FRAME should be equal to 3 (the 3 sensor axes)\n");
//        return;
//    }
//}

/**
 * @brief Return the sign of the number
 * 
 * @param number 
 * @return int 1 if positive (or 0) -1 if negative
 */
//float ei_get_sign(float number) {
//    return (number >= 0.0) ? 1.0 : -1.0;
//}

/**
* @brief      Get data and run inferencing
*
* @param[in]  debug  Get debug info if true
*/
//void loop()
//{
//    ei_printf("\nStarting inferencing in 1 seconds...\n");
//
//    delay(1000);
//
//    ei_printf("Sampling...\n");
//
//    // Allocate a buffer here for the values we'll read from the IMU
//    float buffer[EI_CLASSIFIER_DSP_INPUT_FRAME_SIZE] = { 0 };
//
//    for (size_t ix = 0; ix < EI_CLASSIFIER_DSP_INPUT_FRAME_SIZE; ix += 3) {
//        // Determine the next tick (and then sleep later)
//        uint64_t next_tick = micros() + (EI_CLASSIFIER_INTERVAL_MS * 1000);
//
////        IMU.readAcceleration(buffer[ix], buffer[ix + 1], buffer[ix + 2]);
//        lis.getAcceleration(&buffer[ix], &buffer[ix + 1], &buffer[ix + 2]);
//
//        for (int i = 0; i < 3; i++) {
//            if (fabs(buffer[ix + i]) > MAX_ACCEPTED_RANGE) {
//                buffer[ix + i] = ei_get_sign(buffer[ix + i]) * MAX_ACCEPTED_RANGE;
//            }
//        }
//
//        buffer[ix + 0] *= CONVERT_G_TO_MS2;
//        buffer[ix + 1] *= CONVERT_G_TO_MS2;
//        buffer[ix + 2] *= CONVERT_G_TO_MS2;
//
//        delayMicroseconds(next_tick - micros());
//    }

    // Turn the raw buffer in a signal which we can the classify
//    signal_t signal;
//    int err = numpy::signal_from_buffer(buffer, EI_CLASSIFIER_DSP_INPUT_FRAME_SIZE, &signal);
//    if (err != 0) {
//        ei_printf("Failed to create signal from buffer (%d)\n", err);
//        return;
//    }

    // Run the classifier
//    ei_impulse_result_t result = { 0 };
//
//    err = run_classifier(&signal, &result, debug_nn);
//    if (err != EI_IMPULSE_OK) {
//        ei_printf("ERR: Failed to run classifier (%d)\n", err);
//        return;
//    }

    // print the predictions
//    ei_printf("Predictions ");
//    ei_printf("(DSP: %d ms., Classification: %d ms., Anomaly: %d ms.)",
//        result.timing.dsp, result.timing.classification, result.timing.anomaly);
//    ei_printf(": \n");
//    for (size_t ix = 0; ix < EI_CLASSIFIER_LABEL_COUNT; ix++) {
//        ei_printf("    %s: %.5f\n", result.classification[ix].label, result.classification[ix].value);
//    }
//#if EI_CLASSIFIER_HAS_ANOMALY == 1
//    ei_printf("    anomaly score: %.3f\n", result.anomaly);
//#endif
//
//
//   if (result.classification[1].value > 0.7) {
//    tft.fillScreen(TFT_PURPLE);
//    tft.setFreeFont(&FreeSansBoldOblique12pt7b);
//    tft.drawString("Rest", 20, 80);
//    delay(1000);
//    tft.fillScreen(TFT_WHITE);
//   }
// 
//   if (result.classification[2].value > 0.7) {
//    tft.fillScreen(TFT_RED);
//    tft.setFreeFont(&FreeSansBoldOblique12pt7b);
//    tft.drawString("Jump", 20, 80);
//    delay(1000);
//    tft.fillScreen(TFT_WHITE);
//   }
//
//}
//
//#if !defined(EI_CLASSIFIER_SENSOR) || EI_CLASSIFIER_SENSOR != EI_CLASSIFIER_SENSOR_ACCELEROMETER
//#error "Invalid model for current sensor"
//#endif
